/* /////////////////////////////////////////////////////////////////////////
 * File:        stlsoft/util/minmax.hpp (originally MLFwdEnm.h)
 *
 * Purpose:     Definition of minimum() and maximum() template functions.
 *
 * Created:     11th April 2005
 * Updated:     10th August 2010
 *
 * Home:        http://stlsoft.org/
 *
 * Copyright (c) 2005-2010, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////// */


/** \file stlsoft/util/minmax.hpp
 *
 * \brief [C++ only] Definition of the stlsoft::minimum() and
 *   stlsoft::maximum() function templates
 *   (\ref group__library__utility "Utility" Library).
 */

#ifndef STLSOFT_INCL_STLSOFT_UTIL_HPP_MINMAX
#define STLSOFT_INCL_STLSOFT_UTIL_HPP_MINMAX

#ifndef STLSOFT_DOCUMENTATION_SKIP_SECTION
# define STLSOFT_VER_STLSOFT_UTIL_HPP_MINMAX_MAJOR       2
# define STLSOFT_VER_STLSOFT_UTIL_HPP_MINMAX_MINOR       1
# define STLSOFT_VER_STLSOFT_UTIL_HPP_MINMAX_REVISION    1
# define STLSOFT_VER_STLSOFT_UTIL_HPP_MINMAX_EDIT        15
#endif /* !STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef STLSOFT_INCL_STLSOFT_H_STLSOFT
# include <stlsoft/stlsoft.h>
#endif /* !STLSOFT_INCL_STLSOFT_H_STLSOFT */

/* /////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////
 * Functions
 */

/** \brief Determines the minimum of two values
 *
 * \ingroup group__library__utility
 *
 * \note This is supplied to avoid problems with libraries that \#define \c min
 */
template <ss_typename_param_k T>
inline T const& minimum(T const& lhs, T const& rhs)
{
    return (lhs < rhs) ? lhs : rhs;
}

/** \brief Determines the maximum of two values
 *
 * \ingroup group__library__utility
 *
 * \note This is supplied to avoid problems with libraries that \#define \c min
 */
template <ss_typename_param_k T>
inline T const& maximum(T const& lhs, T const& rhs)
{
    return (lhs < rhs) ? rhs : lhs;
}


#ifndef STLSOFT_DOCUMENTATION_SKIP_SECTION

template <
    ss_typename_param_k T
>
inline T const& maximum(
    T const& v0
,   T const& v1
,   T const& v2
)
{
    return maximum(maximum(v0, v1), v2);
}

template <
    ss_typename_param_k T
>
inline T const& minimum(
    T const& v0
,   T const& v1
,   T const& v2
)
{
    return minimum(minimum(v0, v1), v2);
}

template <
    ss_typename_param_k T
>
inline T const& maximum(
    T const& v0
,   T const& v1
,   T const& v2
,   T const& v3
)
{
    return maximum(maximum(v0, v1), maximum(v2, v3));
}

template <
    ss_typename_param_k T
>
inline T const& minimum(
    T const& v0
,   T const& v1
,   T const& v2
,   T const& v3
)
{
    return minimum(minimum(v0, v1), minimum(v2, v3));
}

template <
    ss_typename_param_k T
>
inline T const& maximum(
    T const& v0
,   T const& v1
,   T const& v2
,   T const& v3
,   T const& v4
)
{
    return maximum(maximum(maximum(v0, v1), maximum(v2, v3)), v4);
}

template <
    ss_typename_param_k T
>
inline T const& minimum(
    T const& v0
,   T const& v1
,   T const& v2
,   T const& v3
,   T const& v4
)
{
    return minimum(minimum(minimum(v0, v1), minimum(v2, v3)), v4);
}

#endif /* !STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////// */

#endif /* !STLSOFT_INCL_STLSOFT_UTIL_HPP_MINMAX */

/* ///////////////////////////// end of file //////////////////////////// */
